"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageContext = void 0;
const utils_1 = require("../utils");
const models_1 = require("../models");
const ChatContext_1 = require("./ChatContext");
class MessageContext extends models_1.Message {
    constructor(api, source) {
        super(source);
        this.api = api;
        this.match = [];
        this.chat = new ChatContext_1.ChatContext(this.api, this.get("chat"));
        this.replyVoice = this.sendAttachWithCaption("voice", true);
        this.replyVideo = this.sendAttachWithCaption("video", true);
        this.replyAudio = this.sendAttachWithCaption("audio", true);
        this.replyPhoto = this.sendAttachWithCaption("photo", true);
        this.replyDocument = this.sendAttachWithCaption("document", true);
        this.sendVoice = this.sendAttachWithCaption("voice");
        this.sendVideo = this.sendAttachWithCaption("video");
        this.sendAudio = this.sendAttachWithCaption("audio");
        this.sendPhoto = this.sendAttachWithCaption("photo");
        this.sendDocument = this.sendAttachWithCaption("document");
    }
    pin(id = this.id, disableNotification = true) {
        return this.execute("pinChatMessage", {
            message_id: id,
            disable_notification: disableNotification,
        });
    }
    unpin(id = this.id) {
        return this.execute("unpinChatMessage", {
            message_id: id,
        });
    }
    unpinAll() {
        return this.execute("unpinAllChatMessages", {});
    }
    async editText(text = this.text, params = {}) {
        var _a;
        let isCaptionEdit = this.hasAttachments;
        params[isCaptionEdit ? "caption" : "text"] = text;
        if (!params.message_id) {
            params.message_id = this.id;
        }
        if (params.message_id === this.id) {
            this.text = text;
            this.keyboard = (_a = params.reply_markup) !== null && _a !== void 0 ? _a : this.keyboard;
        }
        let response = await this.execute(isCaptionEdit ? "editMessageCaption" : "editMessageText", params);
        return typeof response === "object"
            ? new models_1.Message(response.message)
            : undefined;
    }
    async editKeyboard(keyboard, params = {}) {
        params.reply_markup = keyboard;
        if (!params.message_id) {
            params.message_id = this.id;
        }
        if (params.message_id === this.id) {
            this.keyboard = keyboard;
        }
        let response = await this.execute("editMessageReplyMarkup", params);
        return typeof response === "object"
            ? new models_1.Message(response.message)
            : undefined;
    }
    delete(msgId = this.id) {
        return this.execute("deleteMessage", {
            message_id: msgId,
        });
    }
    replyMessage(params) {
        params = typeof params === "string" ? { text: params } : params;
        return this.reply("message", params);
    }
    sendMessage(params) {
        params = typeof params === "string" ? { text: params } : params;
        return this.send("message", params);
    }
    sendAttachWithCaption(type, reply = false) {
        return (source, params) => {
            var _a;
            params = typeof params === "string" ? { caption: params } : params;
            if (reply)
                params["reply_to_message_id"] =
                    (_a = params === null || params === void 0 ? void 0 : params.reply_to_message_id) !== null && _a !== void 0 ? _a : this.id;
            return this.attach(type, source, params);
        };
    }
    async attach(type, source, params = {}) {
        let sourceParams = {
            [type]: source instanceof (models_1.Attachment)
                ? { file_id: source.id }
                : source,
        };
        let response = await this.execute(`send${(0, utils_1.capitalize)(type)}`, Object.assign(Object.assign({}, params), sourceParams), "upload");
        return new models_1.Message(response.message);
    }
    reply(method, params) {
        params.reply_to_message_id = this.id;
        return this.send(method, params);
    }
    async send(method, params) {
        params.chat_id = this.chat.id;
        let response = await this.execute(`send${(0, utils_1.capitalize)(method)}`, params);
        return new models_1.Message(response.message);
    }
    async execute(method, params, apiMethod = "callMethod") {
        params.chat_id = this.chat.id;
        return (await this.api[apiMethod](method, params)).result;
    }
}
exports.MessageContext = MessageContext;
